import { FriksjonBeregningMetadataDto } from "../api/useHentFriksjonsanalyser";
import {
    Bar,
    BarChart,
    CartesianGrid,
    Label,
    Rectangle,
    ResponsiveContainer,
    Tooltip,
    XAxis,
    YAxis,
} from "recharts";
import React, { useMemo, useState } from "react";
import styled from "styled-components";
import { breakpointNettbrett } from "../../utils/constants";
import { Heading } from "../../components/fellesKomponenter/typografi/Heading";
import dayjs, { Dayjs } from "dayjs";
import { ColumnWrapper } from "../../components/fellesKomponenter/wrappers/wrappers";

export const BarPlot = ({ metadata }: { metadata: FriksjonBeregningMetadataDto[] | undefined }) => {
    const [fylkeFilter, setFylkeFilter] = useState<string | null>();
    const [veiFilter, setVeiFilter] = useState<string | null>(null);
    const [startDateFilter, setStartDateFilter] = useState<Dayjs | null>(dayjs("2023-01-01"));
    const [endDateFilter, setEndDateFilter] = useState<Dayjs | null>(null);

    type AggregateDataType = { category: string; data: CountType[] };

    type CountType = { name: string; count: number };

    function aggregateData(
        metadata: FriksjonBeregningMetadataDto[] | undefined,
    ): AggregateDataType[] | undefined {
        if (!metadata) return undefined;

        // Filter the metadata based on the selected filters
        const filteredMetadata = metadata.filter((curr) => {
            if (fylkeFilter && curr.fylke !== fylkeFilter) return false;
            if (veiFilter && curr.vegkategori !== veiFilter) return false;
            if (startDateFilter && dayjs(curr.startDato).isBefore(startDateFilter)) return false;
            if (endDateFilter && dayjs(curr.sluttDato).isAfter(endDateFilter)) return false;
            return true;
        });

        // Aggregate the data
        const fylkeCounts = filteredMetadata.reduce((acc: { [key: string]: number }, curr) => {
            acc[curr.fylke] = (acc[curr.fylke] || 0) + 1;
            return acc;
        }, {});

        const veiCounts = filteredMetadata.reduce((acc: { [key: string]: number }, curr) => {
            acc[curr.vegkategori] = (acc[curr.vegkategori] || 0) + 1;
            return acc;
        }, {});

        const dateCounts = filteredMetadata.reduce((acc: { [key: string]: number }, curr) => {
            const month = dayjs(curr.startDato).format("MM");
            acc[month] = (acc[month] || 0) + 1;
            return acc;
        }, {});

        // Convert the counts objects into arrays of objects
        const fylkeData = Object.entries(fylkeCounts).map(([name, count]) => ({ name, count }));
        const veiData = Object.entries(veiCounts).map(([name, count]) => ({ name, count }));
        const dateData = Object.entries(dateCounts).map(([name, count]) => ({ name, count }));

        return [
            { category: "Fylke", data: fylkeData },
            { category: "Veikategori", data: veiData },
            { category: "Måned", data: dateData },
        ];
    }

    const aggregatedData = useMemo(() => {
        return aggregateData(metadata);
    }, [metadata, fylkeFilter, veiFilter, startDateFilter, endDateFilter]);

    return (
        <div>
            <GrafContainer>
                <CustomHeading styleLevel={"4"} as={"h4"}>
                    Antall målinger
                </CustomHeading>
                <RowContainer>
                    {(aggregatedData || []).map((item, index) => (
                        <ColumnWrapper key={index}>
                            <Heading as={"h5"} styleLevel={"5"}>
                                {item.category}
                            </Heading>
                            <ResponsiveContainer height={360} minWidth={250} key={index}>
                                <BarChart
                                    key={index}
                                    margin={{ top: 50, right: 40, left: 20, bottom: 30 }}
                                    data={item.data}
                                >
                                    <CartesianGrid strokeDasharray="3 3" />
                                    <XAxis dataKey="name">
                                        <Label
                                            value={item.category}
                                            offset={-15}
                                            position="insideBottom"
                                        />
                                    </XAxis>
                                    <YAxis>
                                        <Label
                                            value={"Antall"}
                                            offset={15}
                                            position="insideBottomLeft"
                                            angle={-90}
                                        />
                                    </YAxis>

                                    <Tooltip />
                                    <Bar
                                        dataKey="count"
                                        fill="#8884d8"
                                        activeBar={<Rectangle fill="#544dda" stroke="blue" />}
                                    />
                                </BarChart>
                            </ResponsiveContainer>
                        </ColumnWrapper>
                    ))}
                </RowContainer>
            </GrafContainer>
        </div>
    );
};

const GrafContainer = styled.div`
    @media (max-width: ${breakpointNettbrett}) {
        overflow-x: scroll;
        overflow-y: hidden;
    }
`;

const CustomHeading = styled(Heading)`
    display: flex;
    justify-content: center;
    margin-bottom: 10px;
`;

const RowContainer = styled.div`
    display: flex;
    flex-direction: row;
    flex-wrap: wrap;
`;

export default BarPlot;
